@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,25 +12,19 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from __future__ import with_statement
-
 import ctypes
 import os
 import subprocess
-import sys
 import time
 import signal as signal_module
 
 from robot.utils import (ConnectionCache, abspath, encode_to_system,
-                         decode_output, secs_to_timestr, timestr_to_secs)
+                         decode_output, is_truthy, secs_to_timestr,
+                         timestr_to_secs, IRONPYTHON, JYTHON)
 from robot.version import get_version
 from robot.api import logger
 
 
-if os.sep == '/' and sys.platform.startswith('java'):
-    encode_to_system = lambda string: string
-
-
 class Process(object):
     """Robot Framework test library for running processes.
 
@@ -64,26 +58,32 @@ class Process(object):
 
     = Specifying command and arguments =
 
-    Both `Run Process` and `Start Process` accept the command to execute
-    and all arguments passed to it as separate arguments. This is convenient
-    to use and also allows these keywords to automatically escape possible
-    spaces and other special characters in the command or arguments.
+    Both `Run Process` and `Start Process` accept the command to execute and
+    all arguments passed to the command as separate arguments. This makes usage
+    convenient and also allows these keywords to automatically escape possible
+    spaces and other special characters in commands and arguments. Notice that
+    if a command accepts options that themselves accept values, these options
+    and their values must be given as separate arguments.
 
-    When `running processes in shell`, it is also possible to give the
-    whole command to execute as a single string. The command can then
-    contain multiple commands, for example, connected with pipes. When
-    using this approach the caller is responsible on escaping.
+    When `running processes in shell`, it is also possible to give the whole
+    command to execute as a single string. The command can then contain
+    multiple commands to be run together. When using this approach, the caller
+    is responsible on escaping.
 
     Examples:
-    | `Run Process` | ${progdir}${/}prog.py        | first arg | second         |
-    | `Run Process` | script1.sh arg && script2.sh | shell=yes | cwd=${progdir} |
+    | `Run Process` | ${tools}${/}prog.py | argument | second arg with spaces |
+    | `Run Process` | java | -jar | ${jars}${/}example.jar | --option | value |
+    | `Run Process` | prog.py "one arg" && tool.sh | shell=yes | cwd=${tools} |
+
+    Starting from Robot Framework 2.8.6, possible non-string arguments are
+    converted to strings automatically.
 
     = Process configuration =
 
     `Run Process` and `Start Process` keywords can be configured using
-    optional `**configuration` keyword arguments. Configuration arguments
+    optional ``**configuration`` keyword arguments. Configuration arguments
     must be given after other arguments passed to these keywords and must
-    use syntax like `name=value`. Available configuration arguments are
+    use syntax like ``name=value``. Available configuration arguments are
     listed below and discussed further in sections afterwards.
 
     |  = Name =  |                  = Explanation =                      |
@@ -95,18 +95,18 @@ class Process(object):
     | stderr     | Path of a file where to write standard error.         |
     | alias      | Alias given to the process.                           |
 
-    Note that because `**configuration` is passed using `name=value` syntax,
+    Note that because ``**configuration`` is passed using ``name=value`` syntax,
     possible equal signs in other arguments passed to `Run Process` and
-    `Start Process` must be escaped with a backslash like `name\\=value`.
+    `Start Process` must be escaped with a backslash like ``name\\=value``.
     See `Run Process` for an example.
 
     == Running processes in shell ==
 
-    The `shell` argument specifies whether to run the process in a shell or
+    The ``shell`` argument specifies whether to run the process in a shell or
     not. By default shell is not used, which means that shell specific
-    commands, like `copy` and `dir` on Windows, are not available.
+    commands, like ``copy`` and ``dir`` on Windows, are not available.
 
-    Giving the `shell` argument any non-false value, such as `shell=True`,
+    Giving the ``shell`` argument any non-false value, such as ``shell=True``,
     changes the program to be executed in a shell. It allows using the shell
     capabilities, but can also make the process invocation operating system
     dependent.
@@ -119,13 +119,13 @@ class Process(object):
 
     By default the child process will be executed in the same directory
     as the parent process, the process running tests, is executed. This
-    can be changed by giving an alternative location using the `cwd` argument.
+    can be changed by giving an alternative location using the ``cwd`` argument.
     Forward slashes in the given path are automatically converted to
     backslashes on Windows.
 
     `Standard output and error streams`, when redirected to files,
     are also relative to the current working directory possibly set using
-    the `cwd` argument.
+    the ``cwd`` argument.
 
     Example:
     | `Run Process` | prog.exe | cwd=${ROOT}/directory | stdout=stdout.txt |
@@ -133,10 +133,10 @@ class Process(object):
     == Environment variables ==
 
     By default the child process will get a copy of the parent process's
-    environment variables. The `env` argument can be used to give the
+    environment variables. The ``env`` argument can be used to give the
     child a custom environment as a Python dictionary. If there is a need
     to specify only certain environment variable, it is possible to use the
-    `env:<name>=<value>` format to set or override only that named variables.
+    ``env:<name>=<value>`` format to set or override only that named variables.
     It is also possible to use these two approaches together.
 
     Examples:
@@ -149,19 +149,20 @@ class Process(object):
     By default processes are run so that their standard output and standard
     error streams are kept in the memory. This works fine normally,
     but if there is a lot of output, the output buffers may get full and
-    the program can hang.
+    the program can hang. Additionally on Jython, everything written to
+    these in-memory buffers can be lost if the process is terminated.
 
-    To avoid output buffers getting full, it is possible to use `stdout`
-    and `stderr` arguments to specify files on the file system where to
+    To avoid the above mentioned problems, it is possible to use ``stdout``
+    and ``stderr`` arguments to specify files on the file system where to
     redirect the outputs. This can also be useful if other processes or
     other keywords need to read or manipulate the outputs somehow.
 
-    Given `stdout` and `stderr` paths are relative to the `current working
+    Given ``stdout`` and ``stderr`` paths are relative to the `current working
     directory`. Forward slashes in the given paths are automatically converted
     to backslashes on Windows.
 
     As a special feature, it is possible to redirect the standard error to
-    the standard output by using `stderr=STDOUT`.
+    the standard output by using ``stderr=STDOUT``.
 
     Regardless are outputs redirected to files or not, they are accessible
     through the `result object` returned when the process ends.
@@ -193,8 +194,8 @@ class Process(object):
 
     The keywords that operate on started processes will use the active process
     by default, but it is possible to explicitly select a different process
-    using the `handle` argument. The handle can be the identifier returned by
-    `Start Process` or an `alias` explicitly given to `Start Process` or
+    using the ``handle`` argument. The handle can be the identifier returned by
+    `Start Process` or an ``alias`` explicitly given to `Start Process` or
     `Run Process`.
 
     = Result object =
@@ -209,8 +210,8 @@ class Process(object):
     | rc            | Return code of the process as an integer. |
     | stdout        | Contents of the standard output stream.   |
     | stderr        | Contents of the standard error stream.    |
-    | stdout_path   | Path where stdout was redirected or `None` if not redirected. |
-    | stderr_path   | Path where stderr was redirected or `None` if not redirected. |
+    | stdout_path   | Path where stdout was redirected or ``None`` if not redirected. |
+    | stderr_path   | Path where stderr was redirected or ``None`` if not redirected. |
 
     Example:
     | ${result} =            | `Run Process`         | program               |
@@ -223,28 +224,29 @@ class Process(object):
 
     = Boolean arguments =
 
-    Some keywords accept arguments that are handled as Boolean values.
-    If such an argument is given as a string, it is considered false if it
-    is either empty or case-insensitively equal to `false`. Other strings
-    are considered true regardless what they contain, and other argument
-    types are tested using same
+    Some keywords accept arguments that are handled as Boolean values true or
+    false. If such an argument is given as a string, it is considered false if
+    it is either empty or case-insensitively equal to ``false`` or ``no``.
+    Other strings are considered true regardless their value, and other
+    argument types are tested using same
     [http://docs.python.org/2/library/stdtypes.html#truth-value-testing|rules
     as in Python].
 
     True examples:
     | `Terminate Process` | kill=True     | # Strings are generally true.    |
-    | `Terminate Process` | kill=yes      | # Same as above.                 |
-    | `Terminate Process` | kill=${TRUE}  | # Python `True` is true.         |
+    | `Terminate Process` | kill=yes      | # Same as the above.             |
+    | `Terminate Process` | kill=${TRUE}  | # Python ``True`` is true.       |
     | `Terminate Process` | kill=${42}    | # Numbers other than 0 are true. |
 
     False examples:
-    | `Terminate Process` | kill=False    | # String `False` is false.       |
-    | `Terminate Process` | kill=${EMPTY} | # Empty string is false.         |
-    | `Terminate Process` | kill=${FALSE} | # Python `False` is false.       |
-    | `Terminate Process` | kill=${0}     | # Number 0 is false.             |
+    | `Terminate Process` | kill=False    | # String ``false`` is false.   |
+    | `Terminate Process` | kill=no       | # Also string ``no`` is false. |
+    | `Terminate Process` | kill=${EMPTY} | # Empty string is false.       |
+    | `Terminate Process` | kill=${FALSE} | # Python ``False`` is false.   |
 
     Note that prior to Robot Framework 2.8 all non-empty strings, including
-    `false`, were considered true.
+    ``false``, were considered true. Additionally, ``no`` is considered false
+    only in Robot Framework 2.9 and newer.
 
     = Using with OperatingSystem library =
 
@@ -258,7 +260,7 @@ class Process(object):
     if both libraries are imported, the keywords in the Process library are
     used by default. If there is a need to use the OperatingSystem variants,
     it is possible to use `OperatingSystem.Start Process` syntax or use
-    the `BuiltIn` keyword `Set Library Search Order` to change the priority.
+    the BuiltIn keyword `Set Library Search Order` to change the priority.
 
     Other keywords in the OperatingSystem library can be used freely with
     keywords in the Process library.
@@ -266,18 +268,18 @@ class Process(object):
     = Example =
 
     | ***** Settings *****
-    | Library    Process
+    | Library           Process
     | Suite Teardown    `Terminate All Processes`    kill=True
     |
     | ***** Test Cases *****
     | Example
-    |     `Start Process`    program    arg1   arg2    alias=First
-    |     ${handle} =    `Start Process`    command.sh arg | command2.sh   shell=True    cwd=/path
+    |     `Start Process`    program    arg1    arg2    alias=First
+    |     ${handle} =    `Start Process`    command.sh arg | command2.sh    shell=True    cwd=/path
     |     ${result} =    `Run Process`    ${CURDIR}/script.py
     |     `Should Not Contain`    ${result.stdout}    FAIL
     |     `Terminate Process`    ${handle}
     |     ${result} =    `Wait For Process`    First
-    |     `Should Be Equal As Integers`   ${result.rc}    0
+    |     `Should Be Equal As Integers`    ${result.rc}    0
     """
     ROBOT_LIBRARY_SCOPE = 'GLOBAL'
     ROBOT_LIBRARY_VERSION = get_version()
@@ -291,22 +293,23 @@ class Process(object):
     def run_process(self, command, *arguments, **configuration):
         """Runs a process and waits for it to complete.
 
-        `command` and `*arguments` specify the command to execute and arguments
-        passed to it. See `Specifying command and arguments` for more details.
+        ``command`` and ``*arguments`` specify the command to execute and
+        arguments passed to it. See `Specifying command and arguments` for
+        more details.
 
-        `**configuration` contains additional configuration related to starting
-        processes and waiting for them to finish. See `Process configuration`
-        for more details about configuration related to starting processes.
-        Configuration related to waiting for processes consists of `timeout`
-        and `on_timeout` arguments that have same semantics as with `Wait
-        For Process` keyword. By default there is no timeout, and if timeout
-        is defined the default action on timeout is `terminate`.
+        ``**configuration`` contains additional configuration related to
+        starting processes and waiting for them to finish. See `Process
+        configuration` for more details about configuration related to starting
+        processes. Configuration related to waiting for processes consists of
+        ``timeout`` and ``on_timeout`` arguments that have same semantics as
+        with `Wait For Process` keyword. By default there is no timeout, and
+        if timeout is defined the default action on timeout is ``terminate``.
 
         Returns a `result object` containing information about the execution.
 
-        Note that possible equal signs in `*arguments` must be escaped
-        with a backslash (e.g. `name\\=value`) to avoid them to be passed in
-        as `**configuration`.
+        Note that possible equal signs in ``*arguments`` must be escaped
+        with a backslash (e.g. ``name\\=value``) to avoid them to be passed in
+        as ``**configuration``.
 
         Examples:
         | ${result} = | Run Process | python | -c | print 'Hello, world!' |
@@ -315,8 +318,10 @@ class Process(object):
         | ${result} = | Run Process | ${command} | timeout=1min | on_timeout=continue |
         | ${result} = | Run Process | java -Dname\\=value Example | shell=True | cwd=${EXAMPLE} |
 
-        This command does not change the `active process`.
-        `timeout` and `on_timeout` arguments are new in Robot Framework 2.8.4.
+        This keyword does not change the `active process`.
+
+        ``timeout`` and ``on_timeout`` arguments are new in Robot Framework
+        2.8.4.
         """
         current = self._processes.current
         timeout = configuration.pop('timeout', None)
@@ -339,7 +344,8 @@ class Process(object):
 
         Starting from Robot Framework 2.8.5, processes are started so that
         they create a new process group. This allows sending signals to and
-        terminating also possible child processes.
+        terminating also possible child processes. This is not supported by
+        Jython in general nor by Python versions prior to 2.7 on Windows.
         """
         config = ProcessConfig(**configuration)
         executable_command = self._cmd(command, arguments, config.shell)
@@ -362,9 +368,9 @@ class Process(object):
     def is_process_running(self, handle=None):
         """Checks is the process running or not.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
 
-        Returns `True` if the process is still running and `False` otherwise.
+        Returns ``True`` if the process is still running and ``False`` otherwise.
         """
         return self._processes[handle].poll() is None
 
@@ -372,7 +378,7 @@ class Process(object):
                                   error_message='Process is not running.'):
         """Verifies that the process is running.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
 
         Fails if the process has stopped.
         """
@@ -383,7 +389,7 @@ class Process(object):
                                   error_message='Process is running.'):
         """Verifies that the process is not running.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
 
         Fails if the process is still running.
         """
@@ -394,28 +400,30 @@ class Process(object):
         """Waits for the process to complete or to reach the given timeout.
 
         The process to wait for must have been started earlier with
-        `Start Process`. If `handle` is not given, uses the current
+        `Start Process`. If ``handle`` is not given, uses the current
         `active process`.
 
-        `timeout` defines the maximum time to wait for the process. It is
-        interpreted according to Robot Framework User Guide Appendix
-        `Time Format`, for example, '42', '42 s', or '1 minute 30 seconds'.
+        ``timeout`` defines the maximum time to wait for the process. It can be
+        given in
+        [http://robotframework.org/robotframework/latest/RobotFrameworkUserGuide.html#time-format|
+        various time formats] supported by Robot Framework, for example, ``42``,
+        ``42 s``, or ``1 minute 30 seconds``.
 
-        `on_timeout` defines what to do if the timeout occurs. Possible values
+        ``on_timeout`` defines what to do if the timeout occurs. Possible values
         and corresponding actions are explained in the table below. Notice
         that reaching the timeout never fails the test.
 
-        |  = Value =  |               = Action =               |
-        | `continue`  | The process is left running (default). |
-        | `terminate` | The process is gracefully terminated.  |
-        | `kill`      | The process is forcefully stopped.     |
+        | = Value = |               = Action =               |
+        | continue  | The process is left running (default). |
+        | terminate | The process is gracefully terminated.  |
+        | kill      | The process is forcefully stopped.     |
 
         See `Terminate Process` keyword for more details how processes are
         terminated and killed.
 
         If the process ends before the timeout or it is terminated or killed,
         this keyword returns a `result object` containing information about
-        the execution. If the process is left running, Python `None` is
+        the execution. If the process is left running, Python ``None`` is
         returned instead.
 
         Examples:
@@ -432,7 +440,7 @@ class Process(object):
         | Process Should Be Stopped   |                  |                  |
         | Should Be Equal As Integers | ${result.rc}     | -9               |
 
-        `timeout` and `on_timeout` are new in Robot Framework 2.8.2.
+        ``timeout`` and ``on_timeout`` are new in Robot Framework 2.8.2.
         """
         process = self._processes[handle]
         logger.info('Waiting for process to complete.')
@@ -463,49 +471,48 @@ class Process(object):
     def terminate_process(self, handle=None, kill=False):
         """Stops the process gracefully or forcefully.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
 
-        Waits for the process to stop after terminating it. Returns
-        a `result object` containing information about the execution
-        similarly as `Wait For Process`.
+        By default first tries to stop the process gracefully. If the process
+        does not stop in 30 seconds, or ``kill`` argument is given a true value,
+        (see `Boolean arguments`) kills the process forcefully. Stops also all
+        the child processes of the originally started process.
 
-        On Unix-like machines, by default, first tries to terminate the process
-        group gracefully, but forcefully kills it if it does not stop in 30
-        seconds. Kills the process group immediately if the `kill` argument is
-        given any value considered true. See `Boolean arguments` section for
-        more details about true and false values.
+        Waits for the process to stop after terminating it. Returns a `result
+        object` containing information about the execution similarly as `Wait
+        For Process`.
 
-        Termination is done using `TERM (15)` signal and killing using
-        `KILL (9)`. Use `Send Signal To Process` instead if you just want to
-        send either of these signals without waiting for the process to stop.
+        On Unix-like machines graceful termination is done using ``TERM (15)``
+        signal and killing using ``KILL (9)``. Use `Send Signal To Process`
+        instead if you just want to send either of these signals without
+        waiting for the process to stop.
 
-        On Windows, by default, sends `CTRL_BREAK_EVENT` signal to the process
-        group. If that does not stop the process in 30 seconds, or `kill`
-        argument is given a true value, uses Win32 API function
-        `TerminateProcess()` to kill the process forcefully. Note that
-        `TerminateProcess()` does not kill possible child processes.
+        On Windows graceful termination is done using ``CTRL_BREAK_EVENT``
+        event and killing using Win32 API function ``TerminateProcess()``.
 
+        Examples:
         | ${result} =                 | Terminate Process |     |
         | Should Be Equal As Integers | ${result.rc}      | -15 | # On Unixes |
         | Terminate Process           | myproc            | kill=true |
 
-        *NOTE:* Stopping processes requires the
-        [http://docs.python.org/2/library/subprocess.html|subprocess]
-        module to have working `terminate` and `kill` functions. They were
-        added in Python 2.6 and are thus missing from earlier versions.
-        Unfortunately at least beta releases of Jython 2.7
-        [http://bugs.jython.org/issue1898|do not seem to support them either].
+        Limitations:
+        - Graceful termination is not supported on Windows by Jython nor by
+          Python versions prior to 2.7. Process is killed instead.
+        - Stopping the whole process group is not supported by Jython at all
+          nor by Python versions prior to 2.7 on Windows.
+        - On Windows forceful kill only stops the main process, not possible
+          child processes.
 
         Automatically killing the process if termination fails as well as
         returning a result object are new features in Robot Framework 2.8.2.
         Terminating also possible child processes, including using
-        `CTRL_BREAK_EVENT` on Windows, is new in Robot Framework 2.8.5.
+        ``CTRL_BREAK_EVENT`` on Windows, is new in Robot Framework 2.8.5.
         """
         process = self._processes[handle]
         if not hasattr(process, 'terminate'):
             raise RuntimeError('Terminating processes is not supported '
                                'by this Python version.')
-        terminator = self._kill if is_true(kill) else self._terminate
+        terminator = self._kill if is_truthy(kill) else self._terminate
         try:
             terminator(process)
         except OSError:
@@ -530,7 +537,7 @@ class Process(object):
         if hasattr(os, 'killpg'):
             os.killpg(process.pid, signal_module.SIGTERM)
         elif hasattr(signal_module, 'CTRL_BREAK_EVENT'):
-            if sys.platform == 'cli':
+            if IRONPYTHON:
                 # https://ironpython.codeplex.com/workitem/35020
                 ctypes.windll.kernel32.GenerateConsoleCtrlEvent(
                     signal_module.CTRL_BREAK_EVENT, process.pid)
@@ -558,43 +565,34 @@ class Process(object):
         self.__init__()
 
     def send_signal_to_process(self, signal, handle=None, group=False):
-        """Sends the given `signal` to the specified process.
+        """Sends the given ``signal`` to the specified process.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
 
-        Signal can be specified either as an integer, or anything that can
-        be converted to an integer, or as a signal name. In the latter case it
-        is possible to give the name both with or without a `SIG` prefix,
-        but names are case-sensitive. For example, all the examples below
-        send signal `INT (2)`:
+        Signal can be specified either as an integer as a signal name. In the
+        latter case it is possible to give the name both with or without ``SIG``
+        prefix, but names are case-sensitive. For example, all the examples
+        below send signal ``INT (2)``:
 
         | Send Signal To Process | 2      |        | # Send to active process |
         | Send Signal To Process | INT    |        |                          |
         | Send Signal To Process | SIGINT | myproc | # Send to named process  |
 
+        This keyword is only supported on Unix-like machines, not on Windows.
         What signals are supported depends on the system. For a list of
         existing signals on your system, see the Unix man pages related to
-        signal handling (typically `man signal` or `man 7 signal`).
+        signal handling (typically ``man signal`` or ``man 7 signal``).
 
         By default sends the signal only to the parent process, not to possible
         child processes started by it. Notice that when `running processes in
         shell`, the shell is the parent process and it depends on the system
         does the shell propagate the signal to the actual started process.
-        To send the signal to the whole process group, `group` argument can
-        be set to any true value:
 
-        | Send Signal To Process | TERM  | group=yes |
+        To send the signal to the whole process group, ``group`` argument can
+        be set to any true value (see `Boolean arguments`). This is not
+        supported by Jython, however.
 
-        If you are stopping a process, it is often easier and safer to use
-        `Terminate Process` keyword instead.
-
-        *NOTE:* Sending signals requires the
-        [http://docs.python.org/2/library/subprocess.html|subprocess]
-        module to have working `send_signal` function. It was added
-        in Python 2.6 and are thus missing from earlier versions.
-        How well it will work with forthcoming Jython 2.7 is unknown.
-
-        New in Robot Framework 2.8.2. Support for `group` argument is new
+        New in Robot Framework 2.8.2. Support for ``group`` argument is new
         in Robot Framework 2.8.5.
         """
         if os.sep == '\\':
@@ -602,7 +600,7 @@ class Process(object):
         process = self._processes[handle]
         signum = self._get_signal_number(signal)
         logger.info('Sending signal %s (%d).' % (signal, signum))
-        if is_true(group) and hasattr(os, 'killpg'):
+        if is_truthy(group) and hasattr(os, 'killpg'):
             os.killpg(process.pid, signum)
         elif hasattr(process, 'send_signal'):
             process.send_signal(signum)
@@ -624,23 +622,19 @@ class Process(object):
             raise RuntimeError("Unsupported signal '%s'." % name)
 
     def get_process_id(self, handle=None):
-        """Returns the process ID (pid) of the process.
+        """Returns the process ID (pid) of the process as an integer.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
 
-        Returns the pid assigned by the operating system as an integer.
-        Note that with Jython, at least with the 2.5 version, the returned
-        pid seems to always be `None`.
-
-        The pid is not the same as the identifier returned by
+        Notice that the pid is not the same as the handle returned by
         `Start Process` that is used internally by this library.
         """
         return self._processes[handle].pid
 
     def get_process_object(self, handle=None):
-        """Return the underlying `subprocess.Popen`  object.
+        """Return the underlying ``subprocess.Popen`` object.
 
-        If `handle` is not given, uses the current `active process`.
+        If ``handle`` is not given, uses the current `active process`.
         """
         return self._processes[handle]
 
@@ -648,15 +642,15 @@ class Process(object):
                            stderr=False, stdout_path=False, stderr_path=False):
         """Returns the specified `result object` or some of its attributes.
 
-        The given `handle` specifies the process whose results should be
-        returned. If no `handle` is given, results of the current `active
+        The given ``handle`` specifies the process whose results should be
+        returned. If no ``handle`` is given, results of the current `active
         process` are returned. In either case, the process must have been
         finishes before this keyword can be used. In practice this means
         that processes started with `Start Process` must be finished either
         with `Wait For Process` or `Terminate Process` before using this
         keyword.
 
-        If no other arguments than the optional `handle` are given, a whole
+        If no other arguments than the optional ``handle`` are given, a whole
         `result object` is returned. If one or more of the other arguments
         are given any true value, only the specified attributes of the
         `result object` are returned. These attributes are always returned
@@ -702,14 +696,14 @@ class Process(object):
     def _get_result_attributes(self, result, *includes):
         attributes = (result.rc, result.stdout, result.stderr,
                       result.stdout_path, result.stderr_path)
-        includes = (is_true(incl) for incl in includes)
+        includes = (is_truthy(incl) for incl in includes)
         return tuple(attr for attr, incl in zip(attributes, includes) if incl)
 
     def switch_process(self, handle):
         """Makes the specified process the current `active process`.
 
         The handle can be an identifier returned by `Start Process` or
-        the `alias` given to it explicitly.
+        the ``alias`` given to it explicitly.
 
         Example:
         | Start Process  | prog1    | alias=process1 |
@@ -721,12 +715,11 @@ class Process(object):
         self._processes.switch(handle)
 
     def _process_is_stopped(self, process, timeout):
+        stopped = lambda: process.poll() is not None
         max_time = time.time() + timeout
-        while time.time() <= max_time:
-            if process.poll() is not None:
-                return True
-            time.sleep(0.1)
-        return False
+        while time.time() <= max_time and not stopped():
+            time.sleep(min(0.1, timeout))
+        return stopped()
 
 
 class ExecutionResult(object):
@@ -772,6 +765,8 @@ class ExecutionResult(object):
             return ''
         try:
             return self._format_output(stream.read())
+        except IOError:  # http://bugs.jython.org/issue2218
+            return ''
         finally:
             if stream_path:
                 stream.close()
@@ -809,7 +804,7 @@ class ProcessConfig(object):
         self.cwd = self._get_cwd(cwd)
         self.stdout_stream = self._new_stream(stdout)
         self.stderr_stream = self._get_stderr(stderr, stdout, self.stdout_stream)
-        self.shell = is_true(shell)
+        self.shell = is_truthy(shell)
         self.alias = alias
         self.env = self._construct_env(env, rest)
 
@@ -837,7 +832,8 @@ class ProcessConfig(object):
                        for k, v in env.items())
         for key in extra:
             if not key.startswith('env:'):
-                raise RuntimeError("'%s' is not supported by this keyword." % key)
+                raise RuntimeError("Keyword argument '%s' is not supported by "
+                                   "this keyword." % key)
             if env is None:
                 env = os.environ.copy()
             env[encode_to_system(key[4:])] = encode_to_system(extra[key])
@@ -852,11 +848,15 @@ class ProcessConfig(object):
                   'cwd': self.cwd,
                   'env': self.env,
                   'universal_newlines': True}
-        if hasattr(os, 'setsid') and not sys.platform.startswith('java'):
+        if not JYTHON:
+            self._add_process_group_config(config)
+        return config
+
+    def _add_process_group_config(self, config):
+        if hasattr(os, 'setsid'):
             config['preexec_fn'] = os.setsid
         if hasattr(subprocess, 'CREATE_NEW_PROCESS_GROUP'):
             config['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
-        return config
 
     def __str__(self):
         return encode_to_system("""\
@@ -867,9 +867,3 @@ shell = %r
 alias = %s
 env = %r""" % (self.cwd, self.stdout_stream, self.stderr_stream,
                self.shell, self.alias, self.env))
-
-
-def is_true(argument):
-    if isinstance(argument, basestring) and argument.upper() == 'FALSE':
-        return False
-    return bool(argument)
