@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,23 +12,23 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from __future__ import with_statement
+import copy
 
 from robot import utils
 from robot.errors import DataError
+from robot.model import Keywords, Tags
 from robot.variables import contains_var, is_list_var
 
-from .arguments import (PythonArgumentParser, JavaArgumentParser,
-                        DynamicArgumentParser, ArgumentResolver,
-                        ArgumentMapper, JavaArgumentCoercer)
-from .keywords import Keywords, Keyword
+from .arguments import (ArgumentResolver, ArgumentSpec, ArgumentMapper,
+                        DynamicArgumentParser, JavaArgumentCoercer,
+                        JavaArgumentParser, PythonArgumentParser)
+from .model import Keyword
+from .keywordrunner import KeywordRunner
 from .outputcapture import OutputCapturer
 from .runkwregister import RUN_KW_REGISTER
 from .signalhandler import STOP_SIGNAL_MONITOR
 
 
-
-
 def Handler(library, name, method):
     if RUN_KW_REGISTER.is_run_keyword(library.orig_name, name):
         return _RunKeywordHandler(library, name, method)
@@ -51,22 +51,33 @@ def InitHandler(library, method, docgetter=None):
 
 class _RunnableHandler(object):
     type = 'library'
-    _doc = ''
     _executed_in_dry_run = ('BuiltIn.Import Library',
                             'BuiltIn.Set Library Search Order')
 
-    def __init__(self, library, handler_name, handler_method):
+    def __init__(self, library, handler_name, handler_method, doc=''):
         self.library = library
-        self.name = utils.printable_name(handler_name, code_style=True)
+        name = getattr(handler_method, 'robot_name', None) or handler_name
+        self.name = utils.printable_name(name, code_style=True)
         self.arguments = self._parse_arguments(handler_method)
+        self.pre_run_messages = None
         self._handler_name = handler_name
         self._method = self._get_initial_handler(library, handler_name,
                                                  handler_method)
         self._argument_resolver = self._get_argument_resolver(self.arguments)
+        doc, tags = utils.split_tags_from_doc(doc)
+        self._doc = doc
+        self.tags = self._get_tags_from_attribute(handler_method) + tags
 
     def _parse_arguments(self, handler_method):
         raise NotImplementedError
 
+    def _get_tags_from_attribute(self, handler_method):
+        tags =  getattr(handler_method, 'robot_tags', ())
+        if not utils.is_list_like(tags):
+            raise DataError("Expected tags to list like, got %s."
+                            % utils.type_name(tags))
+        return Tags(tags)
+
     def _get_argument_resolver(self, argspec):
         return ArgumentResolver(argspec)
 
@@ -98,6 +109,9 @@ class _RunnableHandler(object):
         pass
 
     def run(self, context, args):
+        if self.pre_run_messages:
+            for message in self.pre_run_messages:
+                context.output.message(message)
         if context.dry_run:
             return self._dry_run(context, args)
         return self._run(context, args)
@@ -117,8 +131,8 @@ class _RunnableHandler(object):
         return self._run_with_output_captured_and_signal_monitor(runner, context)
 
     def _log_args(self, positional, named):
-        positional = [utils.safe_repr(arg) for arg in positional]
-        named = ['%s=%s' % (utils.unic(name), utils.safe_repr(value))
+        positional = [utils.prepr(arg) for arg in positional]
+        named = ['%s=%s' % (utils.unic(name), utils.prepr(value))
                  for name, value in named.items()]
         return 'Arguments: [ %s ]' % ' | '.join(positional + named)
 
@@ -164,8 +178,8 @@ class _RunnableHandler(object):
 class _PythonHandler(_RunnableHandler):
 
     def __init__(self, library, handler_name, handler_method):
-        _RunnableHandler.__init__(self, library, handler_name, handler_method)
-        self._doc = utils.getdoc(handler_method)
+        _RunnableHandler.__init__(self, library, handler_name, handler_method,
+                                  utils.getdoc(handler_method))
 
     def _parse_arguments(self, handler_method):
         return PythonArgumentParser().parse(handler_method, self.longname)
@@ -202,9 +216,8 @@ class _DynamicHandler(_RunnableHandler):
                  argspec=None):
         self._argspec = argspec
         _RunnableHandler.__init__(self, library, handler_name,
-                                  dynamic_method.method)
+                                  dynamic_method.method, utils.unic(doc or ''))
         self._run_keyword_method_name = dynamic_method.name
-        self._doc = doc is not None and utils.unic(doc) or ''
         self._supports_kwargs = dynamic_method.supports_kwargs
         if argspec and argspec[-1].startswith('**'):
             if not self._supports_kwargs:
@@ -261,15 +274,15 @@ class _RunKeywordHandler(_PythonHandler):
 
     def _dry_run(self, context, args):
         _RunnableHandler._dry_run(self, context, args)
-        keywords = self._get_runnable_dry_run_keywords(context, args)
-        keywords.run(context)
+        keywords = self._get_runnable_dry_run_keywords(args)
+        KeywordRunner(context).run_keywords(keywords)
 
-    def _get_runnable_dry_run_keywords(self, context, args):
-        keywords = Keywords([])
+    def _get_runnable_dry_run_keywords(self, args):
+        keywords = Keywords()
         for keyword in self._get_dry_run_keywords(args):
             if contains_var(keyword.name):
                 continue
-            keywords.add_keyword(keyword)
+            keywords.append(keyword)
         return keywords
 
     def _get_dry_run_keywords(self, args):
@@ -284,7 +297,7 @@ class _RunKeywordHandler(_PythonHandler):
     def _get_run_kw_if_keywords(self, given_args):
         for kw_call in self._get_run_kw_if_calls(given_args):
             if kw_call:
-                yield Keyword(kw_call[0], kw_call[1:])
+                yield Keyword(name=kw_call[0], args=kw_call[1:])
 
     def _get_run_kw_if_calls(self, given_args):
         while 'ELSE IF' in given_args:
@@ -317,7 +330,7 @@ class _RunKeywordHandler(_PythonHandler):
 
     def _get_run_kws_keywords(self, given_args):
         for kw_call in self._get_run_kws_calls(given_args):
-            yield Keyword(kw_call[0], kw_call[1:])
+            yield Keyword(name=kw_call[0], args=kw_call[1:])
 
     def _get_run_kws_calls(self, given_args):
         if 'AND' not in given_args:
@@ -333,24 +346,10 @@ class _RunKeywordHandler(_PythonHandler):
 
     def _get_default_run_kw_keywords(self, given_args):
         index = list(self.arguments.positional).index('name')
-        return [Keyword(given_args[index], given_args[index+1:])]
+        return [Keyword(name=given_args[index], args=given_args[index+1:])]
 
-
-class _XTimesHandler(_RunKeywordHandler):
-
-    def __init__(self, handler, name):
-        _RunKeywordHandler.__init__(self, handler.library, handler.name,
-                                    handler._handler_method)
-        self.name = name
-        self._doc = "*DEPRECATED* Replace X times syntax with 'Repeat Keyword'."
-
-    def run(self, context, args):
-        resolved_times = context.namespace.variables.replace_string(self.name)
-        _RunnableHandler.run(self, context, (resolved_times,) + tuple(args))
-
-    @property
-    def longname(self):
-        return self.name
+    def _run_with_output_captured_and_signal_monitor(self, runner, context):
+        return self._run_with_signal_monitoring(runner, context)
 
 
 class _DynamicRunKeywordHandler(_DynamicHandler, _RunKeywordHandler):
@@ -394,3 +393,48 @@ class _JavaInitHandler(_JavaHandler):
         parser = JavaArgumentParser(type='Test Library')
         signatures = self._get_signatures(handler_method)
         return parser.parse(signatures, self.library.name)
+
+
+class EmbeddedArgsTemplate(object):
+
+    def __init__(self, name_regexp, orig_handler):
+        self.arguments = ArgumentSpec()  # Show empty argument spec for Libdoc
+        self._orig_handler = orig_handler
+        self._name_regexp = name_regexp
+
+    def __getattr__(self, item):
+        return getattr(self._orig_handler, item)
+
+    def matches(self, name):
+        return self._name_regexp.match(name) is not None
+
+    def create(self, name):
+        args = self._name_regexp.match(name).groups()
+        return EmbeddedArgs(name, self.name, args, self._orig_handler)
+
+
+class EmbeddedArgs(object):
+
+    def __init__(self, name, orig_name, embedded_args, orig_handler):
+        self.name = name
+        self.orig_name = orig_name
+        self._embedded_args = embedded_args
+        self._orig_handler = orig_handler
+
+    @property
+    def longname(self):
+        return '%s.%s' % (self.library.name, self.name)
+
+    def __getattr__(self, item):
+        return getattr(self._orig_handler, item)
+
+    def run(self, context, args):
+        if args:
+            raise DataError("Positional arguments are not allowed when using "
+                            "embedded arguments.")
+        return self._orig_handler.run(context, self._embedded_args)
+
+    def __copy__(self):
+        # Needed due to https://github.com/IronLanguages/main/issues/1192
+        return EmbeddedArgs(self.name, self.orig_name, self._embedded_args,
+                            copy.copy(self._orig_handler))
