@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,14 +12,15 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-import os.path
 import copy
+import os.path
 
 from robot.output import LOGGER
-from robot.parsing import ResourceFile
 from robot.errors import FrameworkError
-from robot import utils
+from robot.utils import normpath, seq2str2, is_string
 
+from .builder import ResourceFileBuilder
+from .handlerstore import HandlerStore
 from .testlibraries import TestLibrary
 
 
@@ -46,7 +47,7 @@ class Importer(object):
         if path in self._resource_cache:
             LOGGER.info("Found resource file '%s' from cache" % path)
         else:
-            resource = ResourceFile(path).populate()
+            resource = ResourceFileBuilder().build(path)
             self._resource_cache[path] = resource
         return self._resource_cache[path]
 
@@ -55,7 +56,7 @@ class Importer(object):
         key = (name, positional, named)
         if key in self._library_cache:
             LOGGER.info("Found test library '%s' with arguments %s from cache"
-                        % (name, utils.seq2str2(args)))
+                        % (name, seq2str2(args)))
             return self._library_cache[key]
         lib.create_handlers()
         self._library_cache[key] = lib
@@ -64,22 +65,25 @@ class Importer(object):
 
     def _log_imported_library(self, name, args, lib):
         type = lib.__class__.__name__.replace('Library', '').lower()[1:]
+        listener = ', with listener' if lib.has_listener else ''
         LOGGER.info("Imported library '%s' with arguments %s "
-                    "(version %s, %s type, %s scope, %d keywords)"
-                    % (name, utils.seq2str2(args), lib.version or '<unknown>',
-                       type, lib.scope.lower(), len(lib)))
-        if not lib:
+                    "(version %s, %s type, %s scope, %d keywords%s)"
+                    % (name, seq2str2(args), lib.version or '<unknown>',
+                       type, lib.scope.lower(), len(lib), listener))
+        if not lib and not lib.has_listener:
             LOGGER.warn("Imported library '%s' contains no keywords" % name)
 
     def _copy_library(self, lib, newname):
+        # TODO: This won't work if lib.handlers has kws w/ embedded args.
+        # Need better way to re-initialize libs when imported using WITH NAME.
         libcopy = copy.copy(lib)
         libcopy.name = newname
         libcopy.init_scope_handling()
-        libcopy.handlers = utils.NormalizedDict(ignore=['_'])
-        for handler in lib.handlers.values():
+        libcopy.handlers = HandlerStore(lib.handlers._source)
+        for handler in lib.handlers:
             handcopy = copy.copy(handler)
             handcopy.library = libcopy
-            libcopy.handlers[handler.name] = handcopy
+            libcopy.handlers.add(handcopy)
         return libcopy
 
 
@@ -95,7 +99,7 @@ class ImportCache:
         self._items = []
 
     def __setitem__(self, key, item):
-        if not isinstance(key, (basestring, tuple)):
+        if not is_string(key) and not isinstance(key, tuple):
             raise FrameworkError('Invalid key for ImportCache')
         key = self._norm_path_key(key)
         if key not in self._keys:
@@ -121,10 +125,10 @@ class ImportCache:
 
     def _norm_path_key(self, key):
         if self._is_path(key):
-            return utils.normpath(key)
+            return normpath(key, case_normalize=True)
         if isinstance(key, tuple):
             return tuple(self._norm_path_key(k) for k in key)
         return key
 
     def _is_path(self, key):
-        return isinstance(key, basestring) and os.path.isabs(key) and os.path.exists(key)
+        return is_string(key) and os.path.isabs(key) and os.path.exists(key)
